// tempus 2000.11g
#ifndef AOCONTROLLER_SYSTEM_CLASS
#define AOCONTROLLER_SYSTEM_CLASS
//
// (c) Copyright 1995-2001, MZA Associates Corporation
//

#include "tempus.h"


#include "WFSandRecon.h"
#include "TriggeredWeightedSum.h"
#include "SquareWave.h"

class AOController : public System
{
public:

   // Parameters
   float range;
   DMModel& dmModel;
   int propnxy;
   float propdxy;
   float wfs_wavelength;
   float wfs_ifov;
   double wfs_interval;
   float AO_gain;

   // Subsystems
   WFSandRecon wfsandrecon;

   TriggeredWeightedSum AO_command;

   SquareWave zoverc;


   // Inputs
   Input< WaveTrain > incident;
   Input< Vector<float> > actuatorpositions;

   // Outputs
   Output< Vector<float> > dmcommands;

   AOController(SystemNode *parent, char *name,
                float _range,
                DMModel& _dmModel,
                int _propnxy,
                float _propdxy,
                float _wfs_wavelength,
                float _wfs_ifov,
                double _wfs_interval,
                float _AO_gain) : 
      System(parent, name),
      range(_range),
      dmModel(_dmModel),
      propnxy(_propnxy),
      propdxy(_propdxy),
      wfs_wavelength(_wfs_wavelength),
      wfs_ifov(_wfs_ifov),
      wfs_interval(_wfs_interval),
      AO_gain(_AO_gain),
      wfsandrecon(this, "wfsandrecon",
         dmModel,
         propnxy,
         propdxy,
         wfs_wavelength,
         2,
         wfs_ifov,
         wfs_interval),
      AO_command(this, "AO_command",
         AO_gain,
         1.0),
      zoverc(this, "zoverc",
         (range/speedOfLight),
         1.0,
         1.0e+06,
         1.0e+06),
      incident(this, "incident"),
      actuatorpositions(this, "actuatorpositions"),
      dmcommands(this, "dmcommands")
   {
      AO_command.summand1 <<= wfsandrecon.commands;
      wfsandrecon.incident <<= incident;
      wfsandrecon.on <<= zoverc.boolOutputSignal;
      dmcommands <<= AO_command.sum;
      AO_command.summand2 <<= actuatorpositions;
   }

   // void respondToScheduledEvent (const Event & /* event */);
   // void respondToChangedInputs();
   // void respondToInputWarning (const InputBase* /* input */);
   // void respondToOutputRequest (const OutputBase* /* output */);
};

#endif // AOCONTROLLER_SYSTEM_CLASS
