// tempus 2000.11g
#ifndef WFSANDRECON_SYSTEM_CLASS
#define WFSANDRECON_SYSTEM_CLASS
//
// (c) Copyright 1995-2001, MZA Associates Corporation
//

#include "tempus.h"


#include "HartmannWfsDft.h"
#include "Recon.h"

class WFSandRecon : public System
{
public:

   // Parameters
   DMModel& dmModel;
   int propnxy;
   float propdxy;
   float wfs_wavelength;
   int wfs_nsample;
   float wfs_ifov;
   double wfs_interval;

   // Subsystems
   HartmannWfsDft HWFS;

   Recon recon;


   // Inputs
   Input< WaveTrain > incident;
   Input< bool > on;

   // Outputs
   Output< Vector<float> > commands;

private:      // Setting expressions for unsatisfied inputs/external outputs
   Output< double >  HWFS_exposureInterval;
   Output< double >  HWFS_exposureLength;
   Output< double >  HWFS_sampleInterval;

public:
   WFSandRecon(SystemNode *parent, char *name,
               DMModel& _dmModel,
               int _propnxy,
               float _propdxy,
               float _wfs_wavelength,
               int _wfs_nsample,
               float _wfs_ifov,
               double _wfs_interval) : 
      System(parent, name),
      dmModel(_dmModel),
      propnxy(_propnxy),
      propdxy(_propdxy),
      wfs_wavelength(_wfs_wavelength),
      wfs_nsample(_wfs_nsample),
      wfs_ifov(_wfs_ifov),
      wfs_interval(_wfs_interval),
      HWFS(this, "HWFS",
         wfs_wavelength,
         propnxy,
         propdxy,
         wfs_nsample*16*17 + 1,
         wfs_ifov / wfs_nsample,
         dmModel.xMeasurementWidth(),
         1.0,
         1.0,
         wfs_ifov*16.0 / dmModel.xMeasurementWidth(),
         0.0,
         dmModel.xMeasurementLocations(),
         dmModel.yMeasurementLocations(),
         0.0,
         0.0),
      recon(this, "recon",
         dmModel),
      incident(this, "incident"),
      on(this, "on"),
      commands(this, "commands"),
      HWFS_exposureInterval(this, "HWFS_exposureInterval"),
      HWFS_exposureLength(this, "HWFS_exposureLength"),
      HWFS_sampleInterval(this, "HWFS_sampleInterval")
   {
      recon.slopes <<= HWFS.slopes;
      commands <<= recon.commands;
      HWFS.incident <<= incident;
      HWFS.on <<= on;

      // System's Outputs and Subsystems' Inputs that
      // are not connected, but have setting expressions
      HWFS.exposureInterval <<= HWFS_exposureInterval;
      HWFS.exposureLength <<= HWFS_exposureLength;
      HWFS.sampleInterval <<= HWFS_sampleInterval;
   }

   // void respondToScheduledEvent (const Event & /* event */);
   // void respondToChangedInputs();
   // void respondToInputWarning (const InputBase* /* input */);
   void respondToOutputRequest (const OutputBase* output)
   {
      double t = now();
      if(*output == HWFS_exposureInterval)
      {
         HWFS_exposureInterval = (double)(wfs_interval);
      }
      else if(*output == HWFS_exposureLength)
      {
         HWFS_exposureLength = (double)(1.0e-6);
      }
      else if(*output == HWFS_sampleInterval)
      {
         HWFS_sampleInterval = (double)(-1.0);
      }
   }
};

#endif // WFSANDRECON_SYSTEM_CLASS
