// tempus 2000.11g
#ifndef WINDATMOPATH_SYSTEM_CLASS
#define WINDATMOPATH_SYSTEM_CLASS
//
// (c) Copyright 1995-2001, MZA Associates Corporation
//

#include "tempus.h"


#include "TransverseVelocity.h"
#include "TransverseVelocity.h"
#include "AtmoPath.h"

class WindAtmoPath : public System
{
public:

   // Parameters
   AcsAtmSpec AtmSpec;
   int AtmSeed;
   int propnxy;
   float propdxy;
   Vector<float> vplat;
   Vector<float> vtarg;
   Vector<float> vwind;
   float tmax;
   float Dap;

   // Subsystems
   TransverseVelocity transversevelocity;

   TransverseVelocity transversevelocity2;

   AtmoPath atmosphericpath;


   // Inputs
   Input< WaveTrain > outgoingincident;
   Input< WaveTrain > incomingincident;

   // Outputs
   Output< WaveTrain > outgoingtransmitted;
   Output< WaveTrain > incomingtransmitted;

   WindAtmoPath(SystemNode *parent, char *name,
                AcsAtmSpec _AtmSpec,
                int _AtmSeed,
                int _propnxy,
                float _propdxy,
                Vector<float> _vplat,
                Vector<float> _vtarg,
                Vector<float> _vwind,
                float _tmax,
                float _Dap) : 
      System(parent, name),
      AtmSpec(_AtmSpec),
      AtmSeed(_AtmSeed),
      propnxy(_propnxy),
      propdxy(_propdxy),
      vplat(_vplat),
      vtarg(_vtarg),
      vwind(_vwind),
      tmax(_tmax),
      Dap(_Dap),
      transversevelocity(this, "transversevelocity",
         vwind(0),
         vwind(1),
         0.0,
         0.0),
      transversevelocity2(this, "transversevelocity2",
         -vwind(0),
         -vwind(1),
         0.0,
         0.0),
      atmosphericpath(this, "atmosphericpath",
         AtmSpec,
         AtmSeed,
         propnxy,
         propdxy,
         3*Dap,
         0.05,
         -propnxy*propdxy / 2.0,
         propnxy*propdxy / 2.0 + abs(vplat(0)+vwind(0))*tmax,
         -propnxy*propdxy / 2.0,
         propnxy*propdxy / 2.0 + abs(vplat(1)+vwind(1))*tmax,
         -propnxy*propdxy / 2.0,
         propnxy*propdxy / 2.0 + abs(vtarg(0)+vwind(0))*tmax,
         -propnxy*propdxy / 2.0,
         propnxy*propdxy / 2.0 +abs(vtarg(1)+vwind(1))*tmax,
         propdxy,
         0.0,
         0.0,
         0),
      outgoingincident(this, "outgoingincident"),
      incomingincident(this, "incomingincident"),
      outgoingtransmitted(this, "outgoingtransmitted"),
      incomingtransmitted(this, "incomingtransmitted")
   {
      outgoingtransmitted <<= transversevelocity2.outgoingTransmitted;
      transversevelocity2.incomingIncident <<= incomingincident;
      transversevelocity.outgoingIncident <<= outgoingincident;
      incomingtransmitted <<= transversevelocity.incomingTransmitted;
      transversevelocity2.outgoingIncident <<= atmosphericpath.outgoingTransmitted;
      transversevelocity.incomingIncident <<= atmosphericpath.incomingTransmitted;
      atmosphericpath.outgoingIncident <<= transversevelocity.outgoingTransmitted;
      atmosphericpath.incomingIncident <<= transversevelocity2.incomingTransmitted;
   }

   // void respondToScheduledEvent (const Event & /* event */);
   // void respondToChangedInputs();
   // void respondToInputWarning (const InputBase* /* input */);
   // void respondToOutputRequest (const OutputBase* /* output */);
};

#endif // WINDATMOPATH_SYSTEM_CLASS
